local MinBuild, MaxBuild, Build = 40000, 0, select(4, GetBuildInfo())
if Build < MinBuild or ( MaxBuild > 0 and Build > MaxBuild ) then return end
local AddonName, a = ...
local AddonTitle = select(2, GetAddOnInfo(AddonName))
local PlainAddonTitle = AddonTitle:gsub("|c........", ""):gsub("|r", "")
local L = a.Localize
local s = SpellFlashAddon
local x = s.UpdatedVariables

local c = BittensSpellFlashLibrary2

------------------------------------------------------------------------ common
--local poisons = { 
--    a.spells["Wound Poison"].ID, 
--    a.spells["Instant Poison"].ID,
--    a.spells["Deadly Poison"].ID,
--}

local poisons = { 
    L["Wound Poison"], 
    L["Instant Poison"],
    L["Deadly Poison"],
}

local function IsMissingAPoison()
    local min
    if s.InCombat() then
        min = 0
    else
        min = 5 * 60
    end
    return not s.MainHandItemBuff(poisons, min)
        or not s.OffHandItemBuff(poisons, min)
end

local function getEnergyRegen()
    local inactiveRegen, activeRegen = GetPowerRegen()
    return activeRegen
end

a.spells["Wound Poison"] = {
    Type = "item",
    ID = 10918,
    FlashColor = "yellow",
    CheckFirst = IsMissingAPoison,
}

a.spells["Instant Poison"] = {
    Type = "item",
    ID = 6947,
    FlashColor = "yellow",
    CheckFirst = IsMissingAPoison,
}

a.spells["Deadly Poison"] = {
    Type = "item",
    ID = 2892,
    FlashColor = "yellow",
    CheckFirst = IsMissingAPoison,
}

a.spells["Slice and Dice"] = {
    ID = 5171,
    Buff = 5171,
    BuffUnit = "player",
    EarlyRefresh = 1,
    Unit = "player",
}

----------------------------------------------------------------- assassination
local function DoFinisher()
    local cp = GetComboPoints("player")
    return cp == 5 or (cp == 4 and s.HealthPercent() >= 35)
end

a.spells["Overkill"] = { ID = 58427 }

a.spells["Garrote"] = {
    ID = 703,
}

a.spells["Mutilate"] = {
    ID = 1329,
    CheckFirst = function()
        return GetComboPoints("player") < 4 and s.HealthPercent() >= 35
    end
}

a.spells["Backstab"] = {
    ID = 53,
    CheckFirst = function()
        return GetComboPoints("player") < 5 and s.HealthPercent() < 35
    end
}

a.spells["Envenom"] = {
    ID = 32645,
    CheckFirst = function()
        local rupture = a.spells["Rupture"]
        local refresh = s.MyDebuffDuration(rupture.ID) - rupture.EarlyRefresh
        local regen = getEnergyRegen()
        local untilCap = s.MaxPower("player") - s.Power("player") - 25 - regen
        return DoFinisher()
            and refresh > untilCap / regen
            and (untilCap <= regen * c.GetBusyTime()
                or not c.HasBuff("Envenom"))
    end
}

a.spells["Envenom for SnD"] = {
    ID = 32645,
    CheckFirst = function()
        duration = c.GetBuffDuration("Slice and Dice")
        return duration > .1 and duration < 3
    end
}

a.spells["Rupture"] = {
    ID = 1943,
    MyDebuff = 1943,
    EarlyRefresh = 2,
    CheckFirst = function()
        return DoFinisher()
    end
}

a.spells["Vendetta"] = {
    ID = 79140,
    FlashColor = "yellow",
}

a.spells["Vanish"] = {
    ID = 1856,
    NoGCD = true,
    FlashColor = "yellow",
    CheckFirst = function()
        return not c.HasBuff("Overkill", true)
            and not c.HasMyDebuff("Garrote", true)
            and GetComboPoints("player") < 5
    end
}

------------------------------------------------------------------------ combat
local function SnDIsNext()
    -- only called at 4 or 5 combo points, so no need to consider double combo
    -- point generation
    local SnD = a.spells["Slice and Dice"]
    local timeTillRefresh = s.BuffDuration(SnD.ID, "player") - SnD.EarlyRefresh
    if timeTillRefresh < 0 then
        timeTillRefresh = 0
    end
    return s.Power("player")
             + getEnergyRegen() * (timeTillRefresh + 1)
             - (5 - GetComboPoints("player")) * 39
        < s.MaxPower("player")
end

local function GetEnergy()
    local inactiveRegen, activeRegen = GetPowerRegen()
    return s.Power("player") + activeRegen * c.GetBusyTime()
end

a.spells["Blade Flurry"] = { ID = 13877 }

a.spells["Adrenaline Rush"] = {
    ID = 13750,
    NoGCD = 1,
    FlashColor = "yellow",
    CheckFirst = function()
        return s.Power("player") < 20
    end
}

a.spells["Killing Spree"] = {
    ID = 51690,
    FlashColor = "yellow",
    NoRangeCheck = 1,
--    Override = function()
--        return true
--    end,
    CheckFirst = function()
        return s.Power("player") < 20 and not c.HasBuff("Adrenaline Rush")
    end
}

a.spells["Slice and Dice"] = {
    ID = 5171,
    Buff = 5171,
    BuffUnit = "player",
    EarlyRefresh = 1,
    Unit = "player"
}

a.spells["Expose Armor"] = {
    ID = 8647,
    Debuff = c.ARMOR_DEBUFFS,
    EarlyRefresh = 5,
    Continue = 1,
    FlashColor = "yellow",
    CheckFirst = function()
        return GetComboPoints("player") == 5 and not SnDIsNext()
    end
}

a.spells["Rupture unless SnD"] = {
    ID = 1943,
    MyDebuff = 1943,
    EarlyRefresh = 2,
    RequireDebuff = c.BLEED_DEBUFFS,
    Continue = 1, 
    FlashColor = "yellow",
    CheckFirst = function()
        return GetComboPoints("player") == 5
            and not SnDIsNext()
            and not s.Buff("Blade Flurry", "player")
    end
}

a.spells["Eviscerate"] = {
    ID = 2098,
    CheckFirst = function()
        return GetComboPoints("player") == 5
            and not SnDIsNext()
    end
}

a.spells["Revealing Strike"] = {
    ID = 84617,
    MyDebuff = 84617,
    EarlyRefresh = 2,
    CheckFirst = function()
        return GetComboPoints("player") == 4 and not SnDIsNext()
    end
}

a.spells["Sinister Strike"] = {
    ID = 1752,
    CheckFirst = function()
        local cp = GetComboPoints("player") 
        return cp < 4 or (cp == 4 and SnDIsNext())
    end
}
